package arraybasedDS;

/**
 * @author Luca Castelli Aleardi
 * This interface defines some basic navigational procedures
 */
public interface NavigationInterface {

	/**
	 * Compute the normal to a vertex in the mesh: 
	 * the normal is the average of normals of incident faces
	 * Computations are performed with simple float precision
	 */
	public float[] getNormalFloat(int v);
	
	/**
	 * Compute the degree of a vertex v
	 */
	public int vertexDegree(int v);

	/**
	 * Return the name of the data structure
	 */
	public String name();

	/**
	 * Compute the cross product of two vectors
	 */
	public static float[] crossProduct(float[] a, float[] b) {
		  float[] result=new float[3];
		  result[0]=a[1]*b[2]-a[2]*b[1];
		  result[1]=a[2]*b[0]-a[0]*b[2];
		  result[2]=a[0]*b[1]-a[1]*b[0];
		  return result;
	}

	/**
	 * Compute the normal to the triangle UVW
	 */
	public static float[] computeFaceNormal(float[] pU, float[] pV, float[] pW) {
		float[] uv=new float[3];
		float[] uw=new float[3];
		
		uv[0]=pV[0]-pU[0]; uv[1]=pV[1]-pU[1]; uv[2]=pV[2]-pU[2];
		uw[0]=pW[0]-pU[0]; uw[1]=pW[1]-pU[1]; uw[2]=pW[2]-pU[2];	
		return crossProduct(uv, uw);
	}

}
