package arrays;

/**
 * @author Luca Castelli Aleardi (Ecole Polytechnique, 2016)
 * 
 * An implementation of a stack based on an array of integers
 */
public class ArrayBasedStack implements StackInterface {
    private int top;
    private final int maxSize;
    private int[] A;
    
	/**
	 * Initialize a stack (with a given size)
	 */
    public ArrayBasedStack(int size) {
    	this.maxSize=size;
    	this.A=new int[maxSize];
    	this.top=-1;
    }
    
	/**
	 * Pushes a value onto the top of this stack.
	 */
    public void push(int value) {
        if(top==maxSize-1)
        	throw new Error("Stack error: the stack is full");
        
        this.top++;
        this.A[top]=value;
    }

	/**
	 * Removes the value at the top of this stack and returns that value as the value of this function.
	 */
    public int pop() {
        if(!isEmpty()) {
        	int result=this.A[this.top];
            top=top-1;
            return result;
        }
        else
        	throw new Error("Stack error (pop call): stack empty");
    }

	/**
	 * Returns the value at the top of this stack (with deletion).
	 */
    public int top() {
        if(!isEmpty())
        	return this.A[this.top];
        else
        	throw new Error("Stack error (pop call): stack empty");

    }

	/**
	 * Returns the numbers of elements stored in the stack.
	 */
    public int size() {
    	return this.top+1;
    }

	/**
	 * Check whether the stack is empty
	 */
    public boolean isEmpty() {
    	if(this.top==-1)
    		return true;
    	return false;
    }

	/**
	 * Reset the array: all elements are removed (set to 0)
	 */
    public void reset() {
    	int n=this.maxSize;
    	for(int i=0;i<n;i++) {
    		this.A[i]=0;
    	}
    	
    	this.top=-1;
    }

	/**
	 * Returns a string representing the elements stored in the stack
	 */
    public String toString() {
    	String result="stack: ";
    	if(this.isEmpty())
    		return result+"empty";
    	
    	int n=this.size();
    	for(int i=0;i<n;i++) {
    		result=result+this.A[this.top-i]+" ";
    	}
    	return result;
    }

}
