package arrays;


/** 
 * Implementation of an array storing (32 bits) integer references
 * Service bits are encoded within integer values (no need of auxiliary information)
 **/
public class TableNoAdditionalServiceBits {
	
	public int nElements;
	public int[] elements;

	public static int firstServiceBit=30; 	// position of the first service bit
	public static int secondServiceBit=29;	// position of the second service bit 
	public static int elementMask=~((1<<firstServiceBit)+(1<<secondServiceBit)); // mask for retrieving the integer value stored in an integer of the array

	public TableNoAdditionalServiceBits(int nElements) {
		this.nElements=nElements;
		this.elements=new int[this.nElements];		
	}

	public int elementAt(int index) {
		return (this.elements[index]&elementMask);
	}
	
	public void set(int index, int element) {
		this.elements[index]=element;
	}
	
	public int getFirstServiceBit(int index) {
		  int bit=30; // first service bit is stored at last bit of the reference
		  return ((int)(elements[index] >>> bit)) & 0x01;
	}
	
	public void setFirstServiceBit(int index) {
	    int bit=30;	// first service bit is stored at last position in the reference
	    int bitmask = 1 << bit;
	    elements[index] |= bitmask;		
	}

	public int getSecondServiceBit(int index) {
		  int bit=29; // first service bit is stored at last bit of the reference
		  return ((int)(elements[index] >>> bit)) & 0x01;
	}
	
	public void setSecondServiceBit(int index) {
	    int bit=29;	// first service bit is stored at last position in the reference
	    int bitmask = 1 << bit;
	    elements[index] |= bitmask;		
	}

}
