package arrays;

/**
 * @author Luca Castelli Aleardi
 * Implementation of an array of integer references allowing to store 4 service bits per reference
 * Every integer encodes 2 parts: a reference on 28 bits, plus 4 service bits.
 * Integers are encoded on 32 bits.
 */
public class TableWithFourServiceBits {

	public int nElements;
	public int[] elements;

	public TableWithFourServiceBits(int nElements) {
		this.nElements=nElements;
		this.elements=new int[this.nElements];		
	}

	/**
	 * Return the integer reference stored at a given position
	 */
	public int elementAt(int index) { return (this.elements[index]>>4); }
	
	/**
	 * Set the reference stored at a given position in the array
	 */
	public void set(int index, int element) { this.elements[index]=element<<4; }
	
	/**
	 * Get the value of the first service bit of a reference. The result is either 0 or 1.
	 */
	public int getBit0(int index) { return elements[index] & 0x01; }
	public int getBit1(int index) { return ((int)(elements[index] >>> 1)) & 0x01; }
	public int getBit2(int index) { return ((int)(elements[index] >>> 2)) & 0x01; }
	public int getBit3(int index) { return ((int)(elements[index] >>> 3)) & 0x01; }

	/**
	 * Set to 1 the first service bit, of a given reference.
	 */
	public void setBit0(int index) { elements[index]=elements[index]+1; }
	public void setBit1(int index) { elements[index] += 2; }
	public void setBit2(int index) { elements[index] += 4; }
	public void setBit3(int index) { elements[index] += 8; }

	/** 
	 * Return a string representing the i-th element stored in the table
	 * The result contains an integer value, and the two service bits.
	 **/
	public String elementToString(int i) {
		String result=this.elementAt(i)/3+"(";
		result=result+this.elementAt(i)+",";
		result=result+this.getBit0(i)+",";
		result=result+this.getBit1(i)+",";
		result=result+this.getBit2(i)+",";
		return result+getBit3(i)+")";
	}
	
	public static void main(String[] args) {
		int n=100;
		TableWithFourServiceBits t=new TableWithFourServiceBits(n);
		
		for(int i=0;i<n;i++) {
			t.set(i, 2*i+1);
			if(i%2==0)
				t.setBit0(i);
			if(i%3==0)
				t.setBit1(i);
			if(i%4==0)
				t.setBit2(i);
			if(i%5==0)
				t.setBit3(i);
		}
		
		for(int i=0;i<n;i++)
			System.out.println(t.elementToString(i));
	}

}
